@@ -639,7 +639,7 @@ var LibraryEmbind = {
                 // Looping here to support possible embedded '0' bytes
                 for (var i = 0; i <= length; ++i) {
                     var currentBytePtr = value + 4 + i;
-                    if (HEAPU8[currentBytePtr] == 0 || i == length) {
+                    if (i == length || HEAPU8[currentBytePtr] == 0) {
                         var maxRead = currentBytePtr - decodeStartPtr;
                         var stringSegment = UTF8ToString(decodeStartPtr, maxRead);
                         if (str === undefined) {
@@ -748,7 +748,7 @@ var LibraryEmbind = {
             // Looping here to support possible embedded '0' bytes
             for (var i = 0; i <= length; ++i) {
                 var currentBytePtr = value + 4 + i * charSize;
-                if (HEAP[currentBytePtr >> shift] == 0 || i == length) {
+                if (i == length || HEAP[currentBytePtr >> shift] == 0) {
                     var maxReadBytes = currentBytePtr - decodeStartPtr;
                     var stringSegment = decodeString(decodeStartPtr, maxReadBytes);
                     if (str === undefined) {
@@ -1062,42 +1062,24 @@ var LibraryEmbind = {
 #endif
   },
 
-  $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'],
+  $embind__requireFunction__deps: ['$readLatin1String', '$throwBindingError'
+#if USE_LEGACY_DYNCALLS || !WASM_BIGINT
+    , '$getDynCaller'
+#endif
+  ],
   $embind__requireFunction: function(signature, rawFunction) {
     signature = readLatin1String(signature);
 
-    function makeDynCaller(dynCall) {
-#if DYNAMIC_EXECUTION == 0
-      var argCache = [rawFunction];
-      return function() {
-          argCache.length = arguments.length + 1;
-          for (var i = 0; i < arguments.length; i++) {
-            argCache[i + 1] = arguments[i];
-          }
-          return dynCall.apply(null, argCache);
-      };
-#else
-        var args = [];
-        for (var i = 1; i < signature.length; ++i) {
-            args.push('a' + i);
-        }
-
-        var name = 'dynCall_' + signature + '_' + rawFunction;
-        var body = 'return function ' + name + '(' + args.join(', ') + ') {\n';
-        body    += '    return dynCall(rawFunction' + (args.length ? ', ' : '') + args.join(', ') + ');\n';
-        body    += '};\n';
-
-        return (new Function('dynCall', 'rawFunction', body))(dynCall, rawFunction);
+    function makeDynCaller() {
+#if USE_LEGACY_DYNCALLS || !WASM_BIGINT
+      if (signature.indexOf('j') != -1) {
+        return getDynCaller(signature, rawFunction);
+      }
 #endif
+      return wasmTable.get(rawFunction);
     }
 
-#if MINIMAL_RUNTIME
-    var dc = asm['dynCall_' + signature];
-#else
-    var dc = Module['dynCall_' + signature];
-#endif
-    var fp = makeDynCaller(dc);
-
+    var fp = makeDynCaller();
     if (typeof fp !== "function") {
         throwBindingError("unknown function pointer with signature " + signature + ": " + rawFunction);
     }
@@ -1325,7 +1307,7 @@ var LibraryEmbind = {
                 // assume all fields are present without checking.
                 for (var fieldName in fields) {
                     if (!(fieldName in o)) {
-                        throw new TypeError('Missing field');
+                        throw new TypeError('Missing field:  "' + fieldName + '"');
                     }
                 }
                 var ptr = rawConstructor();
