@@ -220,9 +220,6 @@ var SyscallsLibrary = {
   },
 
   $syscallMmap2__deps: ['$SYSCALLS',
-#if !WASM_BACKEND
-    'memalign', 'memset',
-#endif
 #if FILESYSTEM && SYSCALLS_REQUIRE_FILESYSTEM
     '$FS',
 #endif
@@ -1390,9 +1387,7 @@ var SyscallsLibrary = {
     FS.utime(path, atime, mtime);
     return 0;  
   },
-  __sys_fallocate: function(fd, mode, {{{ defineI64Param('off') }}}, {{{ defineI64Param('len') }}}) {
-    {{{ receiveI64ParamAsI32s('off') }}}
-    {{{ receiveI64ParamAsI32s('len') }}}
+  __sys_fallocate: function(fd, mode, off_low, off_high, len_low, len_high) {
     var stream = SYSCALLS.getStreamFromFD(fd)
     var offset = SYSCALLS.get64(off_low, off_high);
     var len = SYSCALLS.get64(len_low, len_high);
@@ -1565,7 +1560,7 @@ var SyscallsLibrary = {
   fd_sync__sig: 'ii',
   fd_sync: function(fd) {
     var stream = SYSCALLS.getStreamFromFD(fd);
-#if WASM_BACKEND && ASYNCIFY
+#if ASYNCIFY
     return Asyncify.handleSleep(function(wakeUp) {
       var mount = stream.node.mount;
       if (!mount.type.syncfs) {
@@ -1586,233 +1581,9 @@ var SyscallsLibrary = {
       return -stream.stream_ops.fsync(stream);
     }
     return 0; // we can't do anything synchronously; the in-memory FS is already synced to
-#endif // WASM_BACKEND && ASYNCIFY
+#endif // ASYNCIFY
   },
 
-#if !WASM_BACKEND
-  // Fastcomp doesn't support the import_name attribute so we have to alias
-  // the internal musl names to the JS names
-  __syscall1__sig: 'ii',
-  __syscall1: '__sys_exit',
-  __syscall3__sig: 'iiii',
-  __syscall3: '__sys_read',
-  __syscall4__sig: 'iiii',
-  __syscall4: '__sys_write',
-  __syscall5__sig: 'iiii',
-  __syscall5: '__sys_open',
-  __syscall9__sig: 'iii',
-  __syscall9: '__sys_link',
-  __syscall10__sig: 'ii',
-  __syscall10: '__sys_unlink',
-  __syscall12__sig: 'ii',
-  __syscall12: '__sys_chdir',
-  __syscall14__sig: 'iiii',
-  __syscall14: '__sys_mknod',
-  __syscall15__sig: 'iii',
-  __syscall15: '__sys_chmod',
-  __syscall20__sig: 'i',
-  __syscall20: '__sys_getpid',
-  __syscall29__sig: 'i',
-  __syscall29: '__sys_pause',
-  __syscall33__sig: 'iii',
-  __syscall33: '__sys_access',
-  __syscall34__sig: 'ii',
-  __syscall34: '__sys_nice',
-  __syscall36__sig: 'i',
-  __syscall36: '__sys_sync',
-  __syscall38__sig: 'iii',
-  __syscall38: '__sys_rename',
-  __syscall39__sig: 'iii',
-  __syscall39: '__sys_mkdir',
-  __syscall40__sig: 'ii',
-  __syscall40: '__sys_rmdir',
-  __syscall41__sig: 'ii',
-  __syscall41: '__sys_dup',
-  __syscall42__sig: 'ii',
-  __syscall42: '__sys_pipe',
-  __syscall51__sig: 'ii',
-  __syscall51: '__sys_acct',
-  __syscall54__sig: 'iiii',
-  __syscall54: '__sys_ioctl',
-  __syscall57__sig: 'iii',
-  __syscall57: '__sys_setpgid',
-  __syscall60__sig: 'ii',
-  __syscall60: '__sys_umask',
-  __syscall63__sig: 'iii',
-  __syscall63: '__sys_dup2',
-  __syscall64__sig: 'i',
-  __syscall64: '__sys_getppid',
-  __syscall65__sig: 'i',
-  __syscall65: '__sys_getpgrp',
-  __syscall66__sig: 'i',
-  __syscall66: '__sys_setsid',
-  __syscall75__sig: 'iii',
-  __syscall75: '__sys_setrlimit',
-  __syscall77__sig: 'iii',
-  __syscall77: '__sys_getrusage',
-  __syscall83__sig: 'iii',
-  __syscall83: '__sys_symlink',
-  __syscall85__sig: 'iiii',
-  __syscall85: '__sys_readlink',
-  __syscall91__sig: 'iii',
-  __syscall91: '__sys_munmap',
-  __syscall94__sig: 'iii',
-  __syscall94: '__sys_fchmod',
-  __syscall96__sig: 'iii',
-  __syscall96: '__sys_getpriority',
-  __syscall97__sig: 'iiii',
-  __syscall97: '__sys_setpriority',
-  __syscall102__sig: 'iii',
-  __syscall102: '__sys_socketcall',
-  __syscall104__sig: 'iiii',
-  __syscall104: '__sys_setitimer',
-  __syscall114__sig: 'iiiii',
-  __syscall114: '__sys_wait4',
-  __syscall121__sig: 'iii',
-  __syscall121: '__sys_setdomainname',
-  __syscall122__sig: 'ii',
-  __syscall122: '__sys_uname',
-  __syscall125__sig: 'iiii',
-  __syscall125: '__sys_mprotect',
-  __syscall132__sig: 'ii',
-  __syscall132: '__sys_getpgid',
-  __syscall133__sig: 'ii',
-  __syscall133: '__sys_fchdir',
-  __syscall142__sig: 'iiiiii',
-  __syscall142: '__sys__newselect',
-  __syscall144__sig: 'iiii',
-  __syscall144: '__sys_msync',
-  __syscall147__sig: 'ii',
-  __syscall147: '__sys_getsid',
-  __syscall148__sig: 'ii',
-  __syscall148: '__sys_fdatasync',
-  __syscall150__sig: 'iii',
-  __syscall150: '__sys_mlock',
-  __syscall151__sig: 'iii',
-  __syscall151: '__sys_munlock',
-  __syscall152__sig: 'ii',
-  __syscall152: '__sys_mlockall',
-  __syscall153__sig: 'i',
-  __syscall153: '__sys_munlockall',
-  __syscall163__sig: 'iiiiii',
-  __syscall163: '__sys_mremap',
-  __syscall168__sig: 'iiii',
-  __syscall168: '__sys_poll',
-  __syscall178__sig: 'iiii',
-  __syscall178: '__sys_rt_sigqueueinfo',
-  __syscall180__sig: 'iiiiiii',
-  __syscall180: '__sys_pread64',
-  __syscall181__sig: 'iiiiiii',
-  __syscall181: '__sys_pwrite64',
-  __syscall183__sig: 'iii',
-  __syscall183: '__sys_getcwd',
-  __syscall191__sig: 'iii',
-  __syscall191: '__sys_ugetrlimit',
-  __syscall192__sig: 'iiiiiii',
-  __syscall192: '__sys_mmap2',
-  __syscall193__sig: 'iiiii',
-  __syscall193: '__sys_truncate64',
-  __syscall194__sig: 'iiiii',
-  __syscall194: '__sys_ftruncate64',
-  __syscall195__sig: 'iii',
-  __syscall195: '__sys_stat64',
-  __syscall196__sig: 'iii',
-  __syscall196: '__sys_lstat64',
-  __syscall197__sig: 'iii',
-  __syscall197: '__sys_fstat64',
-  __syscall198__sig: 'iiii',
-  __syscall198: '__sys_lchown32',
-  __syscall199__sig: 'i',
-  __syscall199: '__sys_getuid32',
-  __syscall200__sig: 'i',
-  __syscall200: '__sys_getgid32',
-  __syscall201__sig: 'i',
-  __syscall201: '__sys_geteuid32',
-  __syscall202__sig: 'i',
-  __syscall202: '__sys_getegid32',
-  __syscall203__sig: 'iii',
-  __syscall203: '__sys_setreuid32',
-  __syscall204__sig: 'iii',
-  __syscall204: '__sys_setregid32',
-  __syscall205__sig: 'iii',
-  __syscall205: '__sys_getgroups32',
-  __syscall207__sig: 'iiii',
-  __syscall207: '__sys_fchown32',
-  __syscall208__sig: 'iiii',
-  __syscall208: '__sys_setresuid32',
-  __syscall209__sig: 'iiii',
-  __syscall209: '__sys_getresuid32',
-  __syscall210__sig: 'iiii',
-  __syscall210: '__sys_setresgid32',
-  __syscall211__sig: 'iiii',
-  __syscall211: '__sys_getresgid32',
-  __syscall212__sig: 'iiii',
-  __syscall212: '__sys_chown32',
-  __syscall213__sig: 'ii',
-  __syscall213: '__sys_setuid32',
-  __syscall214__sig: 'ii',
-  __syscall214: '__sys_setgid32',
-  __syscall218__sig: 'iiii',
-  __syscall218: '__sys_mincore',
-  __syscall219__sig: 'iiii',
-  __syscall219: '__sys_madvise1',
-  __syscall220__sig: 'iiii',
-  __syscall220: '__sys_getdents64',
-  __syscall221__sig: 'iiii',
-  __syscall221: '__sys_fcntl64',
-  __syscall252__sig: 'ii',
-  __syscall252: '__sys_exit_group',
-  __syscall268__sig: 'iiii',
-  __syscall268: '__sys_statfs64',
-  __syscall269__sig: 'iiii',
-  __syscall269: '__sys_fstatfs64',
-  __syscall272__sig: 'iiiiiiii',
-  __syscall272: '__sys_fadvise64_64',
-  __syscall295__sig: 'iiiii',
-  __syscall295: '__sys_openat',
-  __syscall296__sig: 'iiii',
-  __syscall296: '__sys_mkdirat',
-  __syscall297__sig: 'iiiii',
-  __syscall297: '__sys_mknodat',
-  __syscall298__sig: 'iiiiii',
-  __syscall298: '__sys_fchownat',
-  __syscall300__sig: 'iiiii',
-  __syscall300: '__sys_fstatat64',
-  __syscall301__sig: 'iiii',
-  __syscall301: '__sys_unlinkat',
-  __syscall302__sig: 'iiiii',
-  __syscall302: '__sys_renameat',
-  __syscall303__sig: 'iiiiii',
-  __syscall303: '__sys_linkat',
-  __syscall304__sig: 'iiii',
-  __syscall304: '__sys_symlinkat',
-  __syscall305__sig: 'iiiii',
-  __syscall305: '__sys_readlinkat',
-  __syscall306__sig: 'iiiii',
-  __syscall306: '__sys_fchmodat',
-  __syscall307__sig: 'iiiii',
-  __syscall307: '__sys_faccessat',
-  __syscall308__sig: 'iiiiiii',
-  __syscall308: '__sys_pselect6',
-  __syscall320__sig: 'iiiii',
-  __syscall320: '__sys_utimensat',
-  __syscall324__sig: 'iiiiiii',
-  __syscall324: '__sys_fallocate',
-  __syscall330__sig: 'iiii',
-  __syscall330: '__sys_dup3',
-  __syscall331__sig: 'iii',
-  __syscall331: '__sys_pipe2',
-  __syscall333__sig: 'iiiiii',
-  __syscall333: '__sys_preadv',
-  __syscall334__sig: 'iiiiii',
-  __syscall334: '__sys_pwritev',
-  __syscall337__sig: 'iiiiii',
-  __syscall337: '__sys_recvmmsg',
-  __syscall340__sig: 'iiiii',
-  __syscall340: '__sys_prlimit64',
-  __syscall345__sig: 'iiiiii',
-  __syscall345: '__sys_sendmmsg',
-#endif
 };
 
 var WASI_SYSCALLS = set([
@@ -1824,15 +1595,6 @@ var WASI_SYSCALLS = set([
   'fd_sync',
 ]);
 
-// Fallback for cases where the wasi_interface_version.name prefixing fails,
-// and we have the full name from C. This happens in fastcomp which
-// lacks the attribute to set the import module and base names.
-if (!WASM_BACKEND) {
-  for (var x in WASI_SYSCALLS) {
-    SyscallsLibrary['__wasi_' + x] = x;
-  }
-}
-
 for (var x in SyscallsLibrary) {
   var which = null; // if this is a musl syscall, its number
   var wasi = false;
